/* @(#) samples/c/amqsdsc0.c, remote, p520, p520-aux-CSD05G  1.19 00/10/16 15:00:02 */
/**********************************************************************/
/*                                                                    */
/* Module Name: AMQSDSC0.C                                            */
/*                                                                    */
/* Description: DCE GSS Channel Exit routines (source sample)         */
/* <START_COPYRIGHT>                                                  */
/*  Statement:     Licensed Materials - Property of IBM               */ 
/*                                                                    */      
/*                 04L1773, 5765-B73                                  */
/*                 04L1802, 5639-B42                                  */
/*                 04L1788, 5765-B74                                  */
/*                 04L1816, 5765-B75                                  */
/*                 04L1830, 5639-B43                                  */
/*                 (C) Copyright IBM Corp. 1994, 1998                 */ 
/* <END_COPYRIGHT>                                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*  This file contains the DCE GSS Channel exit routines for MQ.      */
/*                                                                    */
/*  This module contains two main entry points :                      */
/*                                                                    */
/*    MQENTRY DCE_SEC_SCY_CHANNELEXIT (Security)                      */
/*    MQENTRY DCE_SEC_SRM_CHANNELEXIT (Send/Receive/Message)          */
/*                                                                    */
/*  These functions call the various internal routines depending on   */
/*  the reason they were invoked.                                     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Notes:                                                             */
/*   Language:     C                                                  */
/*                                                                    */
/*   This source corresponds to the amqrdsc0 object code except that  */ 
/*   all internal MQSeries function calls have been stripped out.     */
/*                                                                    */
/*   On Windows 95 MQAT_DEFAULT==MQAT_WINDOWS_NT.                     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* User actions:                                                      */
/*   There is no debug/trace/error handling. The places               */
/*   where the user should insert debug code are commented.           */
/*                                                                    */
/*   MUST DO (PC platforms):                                          */
/*   -----------------------                                          */ 
/*   On OS/2, NT and Win95, the user must add                         */
/*   the workpath under which the dce\keytabs directory exists        */ 
/*   in the function acquire_cred. The form of the code required and  */ 
/*   the place to insert it are documented there. The exit will       */
/*   not work unless this change is made.                             */
/*                                                                    */
/**********************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <cmqc.h>
#include <cmqxc.h>

#if MQAT_DEFAULT==MQAT_UNIX
  #include <errno.h>
  #include <sys/types.h>
  #include <unistd.h>
  #include <netinet/in.h>
#endif

#if MQAT_DEFAULT==MQAT_WINDOWS_NT
  #define  TRUE        1
  /* htons is present in later versions of the Microsoft Development Environment eg v5.0.*/
  /* When these are the only ones supported                                              */
  /* the next line and all subsequent references to htons1 can be removed. As currently  */
  /* the code must compile on both older and newer versions of the development           */
  /* environment we need to rename htons to htons1.                                      */
  #define  htons1(x)    (((x<<8)&0xff00) | ((x>>8)&0xff))
  #include <windows.h>
#endif

#if MQAT_DEFAULT==MQAT_OS2
  #define  TRUE        1
  #define  htons(x)    (((x<<8)&0xff00) | ((x>>8)&0xff))
  #include <dce/pthread.h>       /* DCE threads                         */
#endif

#include <dce/gssapi.h>

                                       /* Local Structure definitions */
typedef struct
{
   MQBYTE        expectedSecMsg;
   PMQBYTE       pTargetName;
   gss_name_t    PartnerGSSName;
   gss_cred_id_t credentialHandle;
} SECEXITDATA, MQPOINTER PSECEXITDATA;

/*********************************************************************/
/*                                                                   */
/*    WARNING    WARNING   WARNING   WARNING   WARNING   WARNING     */
/*                                                                   */
/* In the struct below MQ development have had to add 5 bytes of     */
/* padding.                                                          */
/* This is there to overcome the fact that the components of MQSeries*/
/* for NT are compiled with a switch that aligns structs on a 8 byte */
/* boundary. Adding these extra bytes means that the alignment is    */
/* kept correct for other platforms which do not align on an 8 byte  */
/* boundary.                                                         */
/* If a compile option with different alignment is used then this    */
/* struct may well need changing to suit.                            */
/*                                                                   */
/*********************************************************************/
typedef struct
{
   struct
   {
      OM_uint16      mechanism;
      MQBYTE         msgType;
      MQBYTE         padding;
      MQBYTE         padding1;
      MQBYTE         padding2;
      MQBYTE         padding3;
      MQBYTE         padding4;
   }        header;
   MQBYTE   msg;
} DCESECMSG, MQPOINTER PDCESECMSG;



typedef struct
{
   OM_uint16      mechanism;
   MQBYTE         sender;
   gss_ctx_id_t   contextHandle;
} CONTEXTDATA, MQPOINTER PCONTEXTDATA;


/* Local macros                */

#define SECURITYEXITCALL   pParms->ExitId==MQXT_CHANNEL_SEC_EXIT
#define SENDEXITCALL       pParms->ExitId==MQXT_CHANNEL_SEND_EXIT
#define RECEIVEEXITCALL    pParms->ExitId==MQXT_CHANNEL_RCV_EXIT
#define MSGEXITCALL        pParms->ExitId==MQXT_CHANNEL_MSG_EXIT

#define EXPECTINGMSG(x)    ((x==MQXCC_SEND_AND_REQUEST_SEC_MSG)||(x==MQXCC_OK))
#define EXPECTSECMSG(x,y,z) y->expectedSecMsg=EXPECTINGMSG(x)?z:DCE_SEC_NONE

#define UNEXPECTEDMSG(x,y) (x->expectedSecMsg!=y)
#define IAMIDLE(x)         (x->expectedSecMsg==DCE_SEC_NONE)

#define DCEGSS_MECHANISM     (OM_uint16) 0x1d76
#if MQAT_DEFAULT==MQAT_WINDOWS_NT
  #define DCE_SECMECHANISM(x)  (x==htons1(DCEGSS_MECHANISM))
#else
  #define DCE_SECMECHANISM(x)  (x==htons(DCEGSS_MECHANISM))
#endif

/* Types of message flowed to the partner exit */

#define DCE_SEC_NONE         0
#define DCE_SEC_INIT_SEC     1
#define DCE_SEC_ACC_SEC      2
#define DCE_SEC_SEAL         3

/**********************************************************************/
/*                                                                    */
/* Function Name: compareMQNames                                      */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*             Called to compare the                                  */
/*             target partner name which may be present in            */
/*             the ExitData of the                                    */
/*             channel definition with the PartnerName field          */
/*             of the channel exit parameters.                        */
/*                                                                    */
/* Input Parameters:  pExitData                                       */
/*                    pPartnerName                                    */
/*                    ChannelType                                     */
/*                    pChannelName                                    */
/*                                                                    */
/* Output Parameters: None                                            */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/* Exit Normal: MQXCC_OK                                              */
/*                                                                    */
/* Exit Error:  MQXCC_SUPPRESS_FUNCTION     names don't match         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Notes:                                                             */
/*   Restrictions: List any restrictions (or None)                    */
/*                                                                    */
/**********************************************************************/
MQLONG  compareMQNames(PMQCHAR pExitData,
                       PMQCHAR pPartnerName,
                       MQLONG ChannelType,
                       PMQBYTE pChannelName)

{
  int i;

  if ( ChannelType == MQCHT_SVRCONN )
    return MQXCC_OK; /* don't know PartnerName for remote MQ clients */

  if ( (pExitData[0] == ' ') || (pExitData[0] == '\0') )
    return MQXCC_OK; /* no target name */

  for (i = 0; i < MQ_EXIT_DATA_LENGTH; i++)
  {
    /* If all the non-space characters at the starts of the target names */
    /* have matched the check is successful.                             */   

    if ( (pExitData[i] == ' ') && (pPartnerName[i] == ' ') )
       return MQXCC_OK;

    if (pExitData[i] != pPartnerName[i])

    {         /* mismatch of names */

              /* Diagnostic output here */

      return MQXCC_SUPPRESS_FUNCTION;
    }

  } /* end for loop */

  return MQXCC_OK; /* full-length correct target name */

}

/**********************************************************************/
/*                                                                    */
/* Function Name: initSecurityExit                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Called to initialize:                                    */
/*                                                                    */
/*             The SECEXITDATA in the ExitUserArea                    */
/*                                                                    */
/*             The CONTEXTDATA in the ExitData                        */
/*                                                                    */
/* Input Parameters:  pExitData                                       */
/*                    pChannelName                                    */
/*                                                                    */
/* Output Parameters: pSecData                                        */
/*                    pContext                                        */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/* Exit Normal: MQXCC_OK                                              */
/*                                                                    */
/* Exit Error:  MQXCC_SUPPRESS_FUNCTION     unable to allocate memory */
/*                                                                    */
/**********************************************************************/
MQLONG  initSecurityExit(PMQCHAR32 pExitData,
                         PMQBYTE pChannelName,
                         PSECEXITDATA pSecData,
                         PCONTEXTDATA pContext)
{
   PMQBYTE pTargetName = (PMQBYTE)malloc(MQ_EXIT_DATA_LENGTH+1);
   MQBYTE  temp[MQ_EXIT_DATA_LENGTH+1];

                                         /* Allocate work area        */
   if ( pTargetName != NULL )
   {
                                         /* Copy exit data area       */
      memcpy(temp, pExitData, MQ_EXIT_DATA_LENGTH);
      temp[MQ_EXIT_DATA_LENGTH] = '\0';
      memset(pTargetName, 0, MQ_EXIT_DATA_LENGTH+1);
      sscanf((char *)temp, "%s", pTargetName);

                               /* now fill in the pSecData structure  */
      pSecData->expectedSecMsg   = DCE_SEC_NONE;
      pSecData->pTargetName      = pTargetName;
      pSecData->PartnerGSSName   = NULL;
      pSecData->credentialHandle = (gss_cred_id_t)NULL;

      memset(pContext, 0, sizeof(CONTEXTDATA));

      return MQXCC_OK;
   }
   else
   {                                   /* Couldn't allocate storage   */
     /* Diagnostic output here */

     return MQXCC_SUPPRESS_FUNCTION;
   } /* endif */
}



/**********************************************************************/
/*                                                                    */
/* Function Name: savePartnerName                                     */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*          Converts a printable name into a GSS internal name        */
/*          format.                                                   */
/*          Uses the call gss_import_name.                            */
/*                                                                    */
/* Input Parameter:   pPartnerName                                    */
/*                    pChannelName                                    */
/*                                                                    */
/* Output Parameters: *pPartnerGSSName                                */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/**********************************************************************/

MQLONG savePartnerName(gss_name_t *pPartnerGSSName,
                       PMQCHAR48   pPartnerName,
                       PMQBYTE pChannelName)
{

   if ( (*pPartnerName[0] != '\0') && (*pPartnerName[0] != ' ') )
   {
     OM_uint32       major_status = 0;
     OM_uint32       minor_status = 0;

     #if MQAT_DEFAULT==MQAT_OS2
     #pragma pack(4)
     gss_buffer_desc name;
     #pragma pack(1)
     #else
     gss_buffer_desc name;
     #endif
     PMQBYTE         pQmNameStr;
                                       /* space for QM name as string */
     pQmNameStr = (PMQBYTE)malloc( MQ_Q_MGR_NAME_LENGTH + 1 );

     if ( pQmNameStr == NULL )
     {                               /* Couldn't allocate storage   */

       /* Diagnostic output here */

       return MQXCC_SUPPRESS_FUNCTION;
     } /* endif */

     memset( pQmNameStr, 0, MQ_Q_MGR_NAME_LENGTH + 1 );

     strncpy( (char*)pQmNameStr, (char*)pPartnerName, MQ_Q_MGR_NAME_LENGTH );
     strtok( (char*)pQmNameStr, " " ); /* null terminate QM name */

     name.length = strlen((char *)pQmNameStr)+1;
     name.value  = pQmNameStr;

     major_status = gss_import_name ( &minor_status,
                                     &name,
                                     GSS_C_NULL_OID,
                                     pPartnerGSSName );

     free(pQmNameStr);

     if (major_status == GSS_S_COMPLETE)
     {
       return MQXCC_OK;
     }
     else
     {
       /* Diagnostic output here */

       return MQXCC_SUPPRESS_FUNCTION;
     }

   }
   else
   {                            /* No PartnerName */
     /* Diagnostic output here */

     return MQXCC_SUPPRESS_FUNCTION;
   }
}


/**********************************************************************/
/*                                                                    */
/* Function Name: acquire_cred                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*          Acquire credentials by:                                   */
/*                                                                    */
/*          Convert the principal into a GSS internal format          */
/*                                                                    */
/*          Call gss_acquire_cred so the principal can initiate and   */
/*          accept security contexts.                                 */
/*                                                                    */
/* Input Parameters:  pChDef                                          */
/*                    pName                                           */
/*                                                                    */
/* Output Parameters: *pCred_handle                                   */
/*                    *pContextHandle                                 */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/**********************************************************************/

MQLONG acquire_cred(PMQCD         pChDef,
                    gss_cred_id_t *pCred_handle,
                    gss_ctx_id_t  *pContextHandle,
                    PMQBYTE       pName)
{
   OM_uint32       major_status = 0;
   OM_uint32       minor_status = 0;
   OM_uint32       extra_minor_status = 0;
   gss_OID_set     actual_mechs = NULL;
   OM_uint32       actual_time;
   gss_name_t      desired_name = NULL;
   gss_buffer_desc name;
   char            FullPathObjName[512];

   *pContextHandle = GSS_C_NO_CONTEXT;
   if ( pName == NULL )
   { /* no name: default credential on gss_init_sec_context() */
      *pCred_handle = GSS_C_NO_CREDENTIAL;
      return MQXCC_OK;
   }

   /*******************************************************************/
   /* We have been given a name: convert it into GSS                  */
   /* format                                                          */
   /*******************************************************************/
   name.length = strlen((char *)pName)+1;
   name.value  = pName;

   major_status = gss_import_name ( &minor_status,
                                    &name,
                                    GSS_C_NULL_OID,
                                    &desired_name );

   if ( major_status != GSS_S_COMPLETE )
   {                       /* DCE call failed */

     /* Diagnostic output here */

     return MQXCC_SUPPRESS_FUNCTION;
   }

   /*******************************************************************/
   /* Call gssdce_register_acceptor_identity so the subsequent        */
   /* gss_acquire_cred call uses the keytable with the principal's    */   
   /* name.                                                           */   
   /*******************************************************************/
                           
  #if MQAT_DEFAULT==MQAT_OS2 || MQAT_DEFAULT==MQAT_WINDOWS_NT 

     /* Set FullPathObjName to the workpath, ie the directory         */
     /* in which mqs.ini lives.                                       */
     /* For example: strcpy(FullPathObjName,"c:\\mqm");               */
     
     /* Complete the string defining the full path of the keytable */
     strcat(FullPathObjName,"\\dce\\keytabs\\"); 

   #endif

   #if MQAT_DEFAULT==MQAT_UNIX 

     /* UNIX: Setup the string defining the full path of the keytable */
     strcpy(FullPathObjName,"/var/mqm/dce/keytabs/"); 

   #endif 

   strcat(FullPathObjName,(const char*)pName);
   major_status = gssdce_register_acceptor_identity(&minor_status, 
                                                    desired_name,
                                                    NULL,
                                                    FullPathObjName);


   if ( major_status != GSS_S_COMPLETE )
   {                       /* DCE call failed */

     /* Diagnostic output here */

     gss_release_name( &minor_status, &desired_name);
     return MQXCC_SUPPRESS_FUNCTION;

   }

   /*******************************************************************/
   /* Call gss_acquire_cred so the principal can initiate and accept  */
   /* security contexts under the 'desired_name' identity.            */
   /*******************************************************************/
   major_status = gss_acquire_cred( &minor_status, desired_name,
                                    0, GSS_C_NULL_OID_SET, GSS_C_BOTH,
                                    pCred_handle,
                                    &actual_mechs, &actual_time);

   if ( major_status == GSS_S_COMPLETE )
   {
      return MQXCC_OK;
   }

   /**********************************************************/
   /* For an MQ client we tried above to acquire_cred        */      
   /* for the login name of the user who started the client  */
   /* application.                                           */
   /* This works if this login name is also a                */
   /* principal which is defined to DCE.                     */
   /*                                                        */
   /* It is also OK if it fails with a no cred               */
   /* response. In this case we know that the                */
   /* principal is not defined to DCE. We set pCred_handle   */
   /* such that gss_init_sec_context will attempt to use     */
   /* the default DCE credential.                            */
   /*                                                        */
   /* MQ server principals must be defined to DCE.           */
   /**********************************************************/
   if ( (pChDef -> ChannelType == MQCHT_CLNTCONN) &&
        (major_status == GSS_S_NO_CRED) )
   {
      *pCred_handle = GSS_C_NO_CREDENTIAL;
      return MQXCC_OK;
   }

   /* The gss_acquire_cred call failed */

   /* Diagnostic output here */

   gss_release_name( &minor_status, &desired_name);
   return MQXCC_SUPPRESS_FUNCTION;

}



/**********************************************************************/
/*                                                                    */
/* Function Name: create_cred                                         */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*          Create the credentials                                    */
/*                                                                    */
/* Input Parameters: pChDef                                           */
/*                                                                    */
/* Output Parameters:*pCredHandle                                     */
/*                   *pContextHandle                                  */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/**********************************************************************/

MQLONG create_cred(PMQCD          pChDef,
                   gss_cred_id_t *pCredHandle,
                   gss_ctx_id_t  *pContextHandle)
{
   MQLONG rc;
   PMQBYTE pClientLoginName;

   /**********************************************************************/
   /* This sectiona allows for the Windows NT equivalent for getting the */
   /* logged user.                                                       */
   /**********************************************************************/
   #if MQAT_DEFAULT==MQAT_WINDOWS_NT
     #include <windows.h>             /* Windows standard header         */
     #include <winbase.h>             /* GetUserName...                  */
     #define MAX_NAME_LEN 256         /* Maximum User Name length        */
     LPTSTR lpBuffer = (LPTSTR)NULL;  /* Input buffer for Logged on User */
     DWORD  dwSize   = MAX_NAME_LEN+1;/* Maximum buffer size             */
     BOOL   bSuccess = FALSE;         /* Return from GetUserName         */
     BOOL   bRetry   = FALSE;         /* Retry GetUserName call?         */
     LONG   NumRetries = 0;           /* Number of retries so far        */
     DWORD  error;   
   #endif /* MQAT_WINDOWS_NT */
   #if MQAT_DEFAULT==MQAT_UNIX 
     #ifndef LOGNAME_MAX
       #define LOGNAME_MAX 8          /* Maximum logon name length       */
     #endif
     int    getlogin_r_rc;            /* Return value from getlogin_r    */ 
     size_t nameLen = LOGNAME_MAX+1;  /* Length buffer for login name    */
   #endif

   if ( pChDef -> ChannelType == MQCHT_CLNTCONN )
   {
     /******************************************************************/
     /* For an exit running as part of an MQ client, if appropriate for*/
     /* the platform we find out the name that the user who started    */
     /* that client application used when he logged in to the client   */
     /* operating system.                                              */
     /* If this user is also defined as a principal to DCE this        */
     /* is the principal which will be used.                           */
     /* Otherwise the client exit uses the default principal.          */
     /******************************************************************/
     #if MQAT_DEFAULT==MQAT_WINDOWS_NT || MQAT_DEFAULT==MQAT_UNIX
     /* Let's get some memory for the buffer */
       #if MQAT_DEFAULT==MQAT_WINDOWS_NT 
         lpBuffer = (LPTSTR) malloc(MAX_NAME_LEN + 1);
         if (lpBuffer == NULL)
       #elif MQAT_DEFAULT==MQAT_UNIX
          pClientLoginName = (PMQBYTE) malloc(LOGNAME_MAX + 1);
          if ( pClientLoginName == NULL)
       #endif
       {
         /* We failed to allocate memory */
 
         /* Diagnostic output here */

         return MQXCC_SUPPRESS_FUNCTION;
       }
     #endif /* MQAT_DEFAULT==MQAT_WINDOWS_NT || MQAT_DEFAULT==MQAT_UNIX */

     #if MQAT_DEFAULT==MQAT_WINDOWS_NT 

     /**************************************************************/
     /* Get the NT login user name                                 */ 
     /**************************************************************/
     for (bRetry = TRUE, NumRetries = 60;
          (bRetry) && (NumRetries > 0); NumRetries--)
     {
       bRetry = FALSE;
       bSuccess = GetUserName(lpBuffer, &dwSize);
       if (!bSuccess)
       {
         error = GetLastError();
         /**************************************************************/
         /* If Server is busy, wait a second & retry                   */
         /**************************************************************/
         if ((error == RPC_S_SERVER_TOO_BUSY)
            ||(error == RPC_S_OUT_OF_RESOURCES)
            ||(error == RPC_S_PROTOCOL_ERROR)
            ||(error == RPC_S_CALL_FAILED))
         {
           bRetry = TRUE;
           Sleep(1000);
         }
       }
     }
     if (bSuccess)     /* done looping: it either worked or it didn't */
     {
        /* Return was fine, copy the buffer across */
        pClientLoginName = (PMQBYTE) lpBuffer;
     }   
     else
     {
        /* NULL if can't get login name, explicitly set the value     */
        /* to NULL: indicates should try to use DCE default login.    */
        pClientLoginName = (PMQBYTE) NULL;

       /* Diagnostic output here */

     }
     #elif MQAT_DEFAULT==MQAT_OS2
       pClientLoginName = NULL; /* no login name */
     #else    

       /**************************************************************/
       /* Get the UNIX login user name                               */ 
       /**************************************************************/

       /* use getlogin_r as exit can be called on each MQCONN on a threaded client */
       getlogin_r_rc = getlogin_r((char *)pClientLoginName,nameLen); /* get login name */
       if (getlogin_r_rc != 0)
       { 
         /* getlogin_r call failed */

         /* NULL if can't get login name, explicitly set the value     */
         /* to NULL: indicates should try to use DCE default login.    */
         pClientLoginName = (PMQBYTE) NULL;
                  
         /* The location of the error value depends on the POSIX level of the UNIX platform. */
         /* Early POSIX versions return -1 and provide the error value in errno;             */ 
         /* later ones provide the error value on the function call return.                  */  
         /* As these returned error codes are never -1, can test getlogin_r_rc and if it is  */
         /* -1 can assume that we are desling with an early POSIX system.                    */

         if (getlogin_r_rc == -1) getlogin_r_rc = errno; 

         /* now getlogin_r_rc has the error value whichever Posix standard is involved.      */
  
         /* Diagnostic output here */

       }


     #endif

     rc = acquire_cred(pChDef,pCredHandle,pContextHandle,pClientLoginName);

     #if MQAT_DEFAULT==MQAT_WINDOWS_NT
     if (lpBuffer)
     {
        free(lpBuffer);
     }
     #elif MQAT_DEFAULT==MQAT_UNIX
     if (pClientLoginName)
     {
        free(pClientLoginName);
     }
     #endif
   }
   else
   {
     /*****************************************************************/
     /* If we're not on a client find out the name of the Q Manager   */
     /* the Q Manager name is held in pChDef -> QMgrName              */
     /*****************************************************************/
      PMQBYTE pQmName;
                                         /* allocate space for QM name */
      pQmName = (PMQBYTE)malloc( MQ_Q_MGR_NAME_LENGTH + 1 );

      if( pQmName == NULL )
      {                        /* unable to allocate memory */

        /* Diagnostic output here */

        return MQXCC_SUPPRESS_FUNCTION;
      }

      memset( pQmName, 0, MQ_Q_MGR_NAME_LENGTH + 1 );

      strncpy( (char*)pQmName, pChDef -> QMgrName, MQ_Q_MGR_NAME_LENGTH );
      strtok( (char*)pQmName, " " );


     /*****************************************************************/
     /* Having got the queue manager name use it to acquire the DCE   */
     /* GSS credential.                                               */
     /*****************************************************************/
      rc = acquire_cred(pChDef,pCredHandle, pContextHandle, pQmName);

      free(pQmName);

   } /* endif */

   return rc;
}


/**********************************************************************/
/*                                                                    */
/* Function Name: composeSecMsg                                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*          Construct a security message to send                      */
/*                                                                    */
/* Input Parameters: pHeader,                                         */
/*                   headerLength,                                    */
/*                   msgType,                                         */
/*                   msg,                                             */
/*                   msgLength,                                       */
/*                   exitReturnCode,                                  */
/*                   pChannelName.                                    */
/*                                                                    */
/* Output Parameters: pParms,                                         */
/*                    *pExitBufAddr,                                  */
/*                    pExitBufLen.                                    */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/

void composeSecMsg(PMQCXP   pParms,
                   PMQBYTE  *pExitBufAddr,
                   PMQLONG  pExitBufLen,
                   PMQBYTE  pHeader,
                   MQLONG   headerLength,
                   MQBYTE   msgType,
                   PMQBYTE  msg,
                   MQLONG   msgLength,
                   MQLONG   exitReturnCode,
                   PMQBYTE  pChannelName)
{
   MQLONG       length;
   PMQBYTE      buffer;
   PDCESECMSG   pDCE_secMsg;

                                           /* find the length of the data   */
   length = headerLength + sizeof(pDCE_secMsg->header) + msgLength;
                                           /* create a buffer to hold data  */
   buffer = (PMQBYTE)malloc((size_t)length);

   if( buffer != NULL )
   {                                      /* make pDCE_secMsg = the buffer ptr*/
      pDCE_secMsg = (PDCESECMSG)&buffer[headerLength];

      if( (pHeader != NULL) && (headerLength > 0) )
      {                                  /* if there's a header cpy to buffer*/
         memcpy(buffer, pHeader, (size_t)headerLength);
      /* if send exit set byte 9 to zero so receive exit knows it's been */
      /* encrypted */
         if (SENDEXITCALL) buffer[8] = '\0';
      } /* endif */
    #if MQAT_DEFAULT==MQAT_WINDOWS_NT
      pDCE_secMsg->header.mechanism = htons1(DCEGSS_MECHANISM);
    #else
      pDCE_secMsg->header.mechanism = htons(DCEGSS_MECHANISM);
    #endif
      pDCE_secMsg->header.msgType = msgType;

      if( (msg != NULL) && (msgLength > 0) )
      {                                     /* cpy over the data            */
         memcpy(&(pDCE_secMsg->msg), msg, (size_t)msgLength);
      } /* endif */

      *pExitBufAddr = buffer;
      *pExitBufLen  = length;

      pParms->ExitResponse2 = MQXR2_USE_EXIT_BUFFER;
      pParms->ExitResponse = exitReturnCode;
   }
   else
   {                               /* unable to allocate memory       */
     /* Diagnostic output here */

      pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
   } /* endif */
}



/**********************************************************************/
/*                                                                    */
/* Function Name: composeDCE_secTokenMsg                              */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*          Compose a DCE GSS token                                   */
/*                                                                    */
/* Input Parameters: major_status,                                    */
/*                   minor_status,                                    */
/*                   msgType,                                         */
/*                   *pOutputToken,                                   */
/*                   pChannelName.                                    */
/*                                                                    */
/* Output Parameters: pParms,                                         */
/*                    *pOutputMsgAddr,                                */
/*                    pOutputMsgLength.                               */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/

void composeDCE_secTokenMsg(PMQCXP          pParms,
                          OM_uint32       major_status,
                          OM_uint32       minor_status,
                          MQBYTE          msgType,
                          gss_buffer_desc *pOutputToken,
                          PMQBYTE         *pOutputMsgAddr,
                          PMQLONG         pOutputMsgLength,
                          PMQBYTE         pChannelName)
{

   if ( major_status == GSS_S_COMPLETE )
   {
      if ( pOutputToken->length > 0 )
      {
         composeSecMsg(pParms,
                       pOutputMsgAddr,
                       pOutputMsgLength,
                       NULL,
                       0,
                       msgType,
                       pOutputToken->value,
                       pOutputToken->length,
                       MQXCC_SEND_SEC_MSG,
                       pChannelName);
      }
      else                                        /* no token            */
      {
         *pOutputMsgAddr   = NULL;
         *pOutputMsgLength = 0;

         pParms->ExitResponse = MQXCC_OK;
      } /* endif */
   }
   else
   {
      if ( GSS_SUPPLEMENTARY_INFO(major_status) & GSS_S_CONTINUE_NEEDED )
      {
         if ( pOutputToken->length > 0 )
         {
            composeSecMsg(pParms,
                          pOutputMsgAddr,
                          pOutputMsgLength,
                          NULL,
                          0,
                          msgType,
                          pOutputToken->value,
                          pOutputToken->length,
                          MQXCC_SEND_AND_REQUEST_SEC_MSG,
                          pChannelName);
         }
      }
      else                                        /* no token            */
      {                       /* DCE call failed */

        /* Diagnostic output here */

        *pOutputMsgAddr   = NULL;
        *pOutputMsgLength = 0;

        pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
      }
   }
}




/**********************************************************************/
/*                                                                    */
/* Function Name: initSecContext                                      */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*          Initiate Security context between this application and    */
/*          the DCE security server.                                  */
/*                                                                    */
/* Input Parameters: pSecData                                         */
/*                   pInputMsgAddr                                    */
/*                   inputMsgLength                                   */
/*                   pChannelName                                     */
/*                                                                    */
/* Output Parameters: pParms                                          */
/*                    *pOutputMsgAddr                                 */
/*                    pOutputMsgLength                                */
/*                                                                    */
/* InOut Parameters:  pContextData                                    */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/

void initSecContext(PMQCXP       pParms,
                    PSECEXITDATA pSecData,
                    PCONTEXTDATA pContextData,
                    PMQBYTE      pInputMsgAddr,
                    MQLONG       inputMsgLength,
                    PMQBYTE      *pOutputMsgAddr,
                    PMQLONG      pOutputMsgLength,
                    PMQBYTE      pChannelName)
{
   gss_buffer_desc output_token = { 0, NULL };
   gss_OID         mech_type    = GSS_C_NULL_OID;
   gss_OID         actual_mech_type;
   int             req_flags    = GSS_C_MUTUAL_FLAG;
   int             ret_flags;
   OM_uint32       time_req     = 0;
   OM_uint32       time_rec     = 0;
   gss_buffer_desc input_token;
   gss_buffer_t    pInput_Token;
   OM_uint32       major_status = 0;
   OM_uint32       minor_status = 0;

   if ( inputMsgLength == 0 )   /* initialize the input token */
     pInput_Token = GSS_C_NO_BUFFER;
   else
   {
     input_token.length = inputMsgLength;
     input_token.value  = pInputMsgAddr;
     pInput_Token = &input_token;
   }
                                        /*  initiate a security context  */
   major_status = gss_init_sec_context(&minor_status,
                                       pSecData->credentialHandle,
                                       &(pContextData->contextHandle),
                                       pSecData->PartnerGSSName,
                                       mech_type, req_flags, time_req,
                                       NULL, pInput_Token,
                                       &actual_mech_type, &output_token,
                                       &ret_flags, &time_rec);

   /* major_status is checked in composeDCE_secTokenMsg */

   composeDCE_secTokenMsg(pParms,
                        major_status,
                        minor_status,
                        DCE_SEC_INIT_SEC,
                        &output_token,
                        pOutputMsgAddr,
                        pOutputMsgLength,
                        pChannelName);

   gss_release_buffer ( &minor_status, &output_token );
}


/**********************************************************************/
/*                                                                    */
/* Function Name: acceptSecContext                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Establist a remotely initiated security context between  */
/*           this application and a remote Peer.                      */
/*                                                                    */
/* Input Parameters: pSecData,                                        */
/*                   pInputMsgAddr,                                   */
/*                   inputMsgLength.                                  */
/*                   pChannelName                                     */
/*                                                                    */
/* Output Parameters: pParms,                                         */
/*                    pOutputMsgAddr,                                 */
/*                    pOutputMsgLength.                               */
/*                                                                    */
/* InOut Parameters:  pContextData.                                   */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/* Notes:                                                             */
/*   Restrictions: List any restrictions (or None)                    */
/*                                                                    */
/**********************************************************************/

void acceptSecContext(PMQCXP       pParms,
                      PSECEXITDATA pSecData,
                      PCONTEXTDATA pContextData,
                      PMQBYTE      pInputMsgAddr,
                      MQLONG       inputMsgLength,
                      PMQBYTE      *pOutputMsgAddr,
                      PMQLONG      pOutputMsgLength,
                      PMQBYTE      pChannelName)
{
   gss_buffer_desc   output_token    = { 0, NULL };
   int               context_support = 0;
   OM_uint32         context_time    = 0;
   gss_cred_id_t     delegated_cred_handle;
   OM_uint32         major_status    = 0;
   OM_uint32         minor_status    = 0;
   gss_OID           mech_type       = GSS_C_NULL_OID;
   gss_buffer_desc   input_token;

   input_token.length = inputMsgLength;
   input_token.value  = pInputMsgAddr;

   major_status = gss_accept_sec_context( &minor_status,
                                          &(pContextData->contextHandle),
                                          pSecData->credentialHandle,
                                          &input_token, NULL,
                                          &(pSecData->PartnerGSSName),
                                          &mech_type, &output_token, &context_support,
                                          &context_time, &delegated_cred_handle);

   /* major_status is checked in composeDCE_secTokenMsg */

   composeDCE_secTokenMsg(pParms,
                        major_status,
                        minor_status,
                        DCE_SEC_ACC_SEC,
                        &output_token,
                        pOutputMsgAddr,
                        pOutputMsgLength,
                        pChannelName);

   gss_release_buffer ( &minor_status, &output_token );
}



/**********************************************************************/
/*                                                                    */
/* Function Name: terminateSecurityExit                               */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Terminate the security context at end of the security    */
/*           exit call.                                               */
/*                                                                    */
/* Input Parameters: pContext,                                        */
/*                                                                    */
/* Output Parameters: pParms.                                         */
/*                                                                    */
/* InOut Parameters:  pSecData.                                       */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/
void terminateSecurityExit(PMQCXP pParms,
                           PSECEXITDATA pSecData,
                           PCONTEXTDATA pContext)
{
   OM_uint32         minor_status    = 0;
   OM_uint32         major_status    = 0;
   gss_buffer_desc   output_token    = {0, NULL};

   if ( pSecData->pTargetName != NULL )
   {
      free(pSecData->pTargetName);
      pSecData->pTargetName = NULL;
   } /* endif */

   if ( pSecData->PartnerGSSName != NULL )
   {
      major_status = gss_release_name(&minor_status, &(pSecData->PartnerGSSName));
   } /* endif */

   if ( pContext->contextHandle != (gss_ctx_id_t)NULL )
   {


      major_status = gss_delete_sec_context(&minor_status,
                                            &(pContext->contextHandle),
                                            &output_token);

      if ( major_status == GSS_S_COMPLETE )
      {
         gss_release_buffer(&minor_status, &output_token);
      } /* endif */
   }

   if ( pSecData->credentialHandle != (gss_cred_id_t)NULL )
   {
      major_status = gss_release_cred(&minor_status, &(pSecData->credentialHandle));
   } /* endif */

   pParms->ExitResponse = MQXCC_OK;

}


/**********************************************************************/
/*                                                                    */
/* Function Name: compareNames                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Compare the Target and Partner names                     */
/*                                                                    */
/* Input Parameters: pTargetName                                      */
/*                   pPartnerName                                     */
/*                                                                    */
/* Output Parameters: None                                            */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/**********************************************************************/

MQLONG compareNames(PMQBYTE pTargetName,
                    PMQBYTE pPartnerName)
{
   MQLONG rc;

   if ( (pPartnerName != NULL) && (strlen((char *)pPartnerName) > 0) )
   {
      if ( *pTargetName == '\0' )
      {
         rc = MQXCC_OK;
      }
      else
      {
         if ( strcmp((char *)pTargetName, (char *)pPartnerName) == 0 )
         {
           rc = MQXCC_OK;
         }
         else
         {
           rc = MQXCC_SUPPRESS_FUNCTION;
         }
      } /* endif */
   }
   else
   {
      rc = MQXCC_SUPPRESS_FUNCTION;
      /* diagnostics are output from the calling routine */

   } /* endif */

   return rc;
}



/**********************************************************************/
/*                                                                    */
/* Function Name: verifyTargetName                                    */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Verify the target name against the remote partner name.  */
/*           This allows restricted channels to be supported.         */
/*           It is only done in this routine for SVRCONN channels     */
/*           because in all other cases the remote partner name is    */
/*           known when the exit is first called and is checked using */
/*           compareMQNames.                                          */
/*                                                                    */
/* Input Parameters: pParms                                           */
/*                                                                    */
/* Output Parameters: pChDef.                                         */
/*                                                                    */
/* InOut Parameters:  None.                                           */
/*                                                                    */
/* Returns: MQLONG    return code value                               */
/*                                                                    */
/**********************************************************************/

MQLONG verifyTargetName(PMQCXP pParms, PMQCD pChDef)
{
   OM_uint32       major_status  = 0;
   OM_uint32       minor_status  = 0;
   gss_OID         nameType      = GSS_C_NULL_OID;
   gss_buffer_desc myFullDCEName = {0, NULL};
   PSECEXITDATA pSecData         = (PSECEXITDATA)&(pParms->ExitUserArea);

   major_status = gss_display_name( &minor_status, pSecData->PartnerGSSName,
                                    &myFullDCEName, &nameType );

   if ( major_status == GSS_S_COMPLETE )
   {
      PMQBYTE FullDCEName;

      FullDCEName = (PMQBYTE)malloc(myFullDCEName.length + 1);

      if ( FullDCEName != NULL )
      {
         MQLONG  retCode;
         PMQBYTE ClientUserid;
         int ch = '/';

         memcpy(FullDCEName, myFullDCEName.value, myFullDCEName.length);
         FullDCEName[myFullDCEName.length] = '\0';

         ClientUserid = (PMQBYTE)strrchr((char*)FullDCEName, ch);
         ClientUserid++;   /* step over / character */

         retCode = compareNames(pSecData->pTargetName, ClientUserid);

         /*************************************************************/
         /* This is a SVRCONN Channel. If there is no previously      */
         /* defined MCAUserIdentifier then copy the partner name into */
         /* the MCAUserIdentifier.                                    */
         /*************************************************************/
         if ( retCode == MQXCC_OK                     &&
              (pChDef -> MCAUserIdentifier[0]  == ' ' ||
               pChDef -> MCAUserIdentifier[0]  == '\0'))
         {
            size_t length;

            length = strlen((char *)ClientUserid);
            length = (length > MQ_USER_ID_LENGTH) ? MQ_USER_ID_LENGTH : length;

            memset(&(pChDef->MCAUserIdentifier[0]), 0, MQ_USER_ID_LENGTH);
            strncpy(&(pChDef->MCAUserIdentifier[0]), (char *)ClientUserid, length);
         }  /* endif */

         if ( retCode != MQXCC_OK )
         {                       /* Names don't match */

           /* Diagnostic output here */

         }

         free(FullDCEName);
         gss_release_buffer(&minor_status, &myFullDCEName);

         return retCode;
      }
      else    /* malloc didn't work */
      {
        /* Diagnostic output here */

        return MQXCC_SUPPRESS_FUNCTION;
      }
   }
   else               /* gss_display_name DCE call failed */
   {
     /* Diagnostic output here */

     return MQXCC_SUPPRESS_FUNCTION;
   }
}


/**********************************************************************/
/*                                                                    */
/* Function Name: processSecurityMessage                              */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Process a security message                               */
/*           process the message dependent upon the value in :-       */
/*                                      pInMsgAddr->header.msgType    */
/*                                                                    */
/* Input Parameters: pChDef                                           */
/*                   pInMsgAddr                                       */
/*                   inMsgLength                                      */
/*                                                                    */
/* Output Parameters:pOutMsgAddr                                      */
/*                   pOutMsgLength                                    */
/*                                                                    */
/* InOut Parameters:  pParms.                                         */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/

void processSecurityMessage(PMQCXP     pParms,
                            PMQCD      pChDef,
                            PDCESECMSG pInMsgAddr,
                            MQLONG     inMsgLength,
                            PMQPTR     pOutMsgAddr,
                            PMQLONG    pOutMsgLength)
{
   PMQBYTE      pMyName = NULL;
   PSECEXITDATA pSecData;
   PCONTEXTDATA pContext;

   pSecData = (PSECEXITDATA)&(pParms->ExitUserArea);
   pContext = (PCONTEXTDATA)&(pChDef->SecurityUserData);

   if ( (!DCE_SECMECHANISM(pInMsgAddr->header.mechanism)) ||
        (UNEXPECTEDMSG(pSecData, pInMsgAddr->header.msgType)) )
   {
     /* Invalid message received */

     if ( IAMIDLE(pSecData) )
     {
       pParms->ExitResponse = MQXCC_OK;
       return;
     }
     else
     {
       /* Diagnostic output here */

       pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;

       return;

     }  /* endif */

   } /* endif */

   switch ( pInMsgAddr->header.msgType )
   {
      case DCE_SEC_INIT_SEC:

      /* acceptor: need to check partner name and create the credentials if first time through */
              if ( pSecData->credentialHandle == (gss_cred_id_t)NULL )
              {
                /* if necessary and possible check target name against partner name in exit parameters */
                pParms->ExitResponse =
                   compareMQNames((PMQCHAR)&(pParms->ExitData[0]),
                                  (PMQCHAR)&(pParms->PartnerName[0]),
                                  pChDef->ChannelType,
                                  (PMQBYTE)&(pChDef->ChannelName));

                if ( pParms->ExitResponse == MQXCC_OK )
                  pParms-> ExitResponse =
                    create_cred( pChDef,
                                 &(pSecData->credentialHandle),
                                 &(pContext->contextHandle) );
              }

              if ( pParms->ExitResponse == MQXCC_OK )
                acceptSecContext(pParms,
                                 pSecData,
                                 pContext,
                                 (MQBYTE *)&(pInMsgAddr->msg),
                                 inMsgLength-sizeof(pInMsgAddr->header.msgType),
                                 (PMQBYTE *)pOutMsgAddr,
                                 pOutMsgLength,
                                 (PMQBYTE)&(pChDef->ChannelName));
              break;
      case DCE_SEC_ACC_SEC:
              initSecContext(pParms,
                             pSecData,
                             pContext,
                             (MQBYTE *)&(pInMsgAddr->msg),
                             inMsgLength-sizeof(pInMsgAddr->header.msgType),
                             (PMQBYTE *)pOutMsgAddr,
                             pOutMsgLength,
                             (PMQBYTE)&(pChDef->ChannelName));
              break;
   } /* endswitch */

   if ( (pParms->ExitResponse == MQXCC_OK) ||
        (pParms->ExitResponse == MQXCC_SEND_SEC_MSG) ) /* fully authorized */
   {

      if ( pChDef -> ChannelType == MQCHT_SVRCONN )
      {
        /* only do restricted channel check here if partner is an MQ client */

        if ( verifyTargetName(pParms, pChDef) != MQXCC_OK )
        {
          /* Diagnostics are output from verifyTargetName */
          pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
          return;
        }
      }  /* done verifyTargetName if needed */

      EXPECTSECMSG(pParms->ExitResponse,
                   pSecData,
                   DCE_SEC_NONE);

      pContext->mechanism = DCEGSS_MECHANISM;  /* for snd/rcv/msg exits */


   } /* end of code run when this partner is fully authorized */
}




/**********************************************************************/
/*                                                                    */
/* Function Name: DCE_SEC_SCY_CHANNELEXIT                             */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Main entry point for channel security exit               */
/*           Check to se that this has been called as a security exit */
/*           Then switch to whatever is defined by pParms->ExitReason */
/*                                                                    */
/* Input Parameters: pAgentBufferLength  UNUSED                       */
/*                                                                    */
/* Output Parameters: None                                            */
/*                                                                    */
/* InOut Parameters: pChannelExitParms   Channel exit parameter block */
/*                   pChannelDefinition  Channel definition           */
/*                   pDataLength         Length of data               */
/*                   pAgentBuffer        Agent buffer                 */
/*                   pExitBufferLength   Length of exit buffer        */
/*                   pExitBufferAddr     Address of exit buffer       */
/*                                                                    */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/

void MQENTRY DCE_SEC_SCY_CHANNELEXIT (
     PMQVOID  pChannelExitParms,   /* Channel exit parameter block */
     PMQVOID  pChannelDefinition,  /* Channel definition           */
     PMQLONG  pDataLength,         /* Length of data               */
     PMQLONG  pAgentBufferLength,  /* Length of agent buffer       */
     PMQVOID  pAgentBuffer,        /* Agent buffer                 */
     PMQLONG  pExitBufferLength,   /* Length of exit buffer        */
     PMQPTR   pExitBufferAddr)     /* Address of exit buffer       */
{
   PMQCXP   pParms = (PMQCXP)pChannelExitParms;
   PMQCD    pChDef = (PMQCD)pChannelDefinition;
   PSECEXITDATA pSecData;
   PCONTEXTDATA pContext;

   pSecData = (PSECEXITDATA)&(pParms->ExitUserArea);
   pContext = (PCONTEXTDATA)&(pChDef->SecurityUserData);

                                         /* Invoked as security exit? */
   if ( !(SECURITYEXITCALL) )
   {
     /* Diagnostic output here */

     pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;
     return;
   }
   else
   {
      /* free buffer used to pass back message on previous exit call */
      if ( (pExitBufferAddr != NULL) && (*pExitBufferAddr != NULL) )
      {
         free(*pExitBufferAddr);
         *pExitBufferAddr = NULL;
         *pExitBufferLength = 0;
      } /* endif */
   } /* endif */

   /* If OS/2: turn this thread into a pthread */
   #if MQAT_DEFAULT==MQAT_OS2
     pthread_pseudo_thread_base_start();
   #endif

     /*******************************************************************/
     /* now switch to what ever function we were called to do           */
     /*******************************************************************/
   switch ( pParms->ExitReason )
   {
      case MQXR_INIT:
              pParms->ExitResponse =
                 initSecurityExit(&(pParms->ExitData),
                                  (PMQBYTE)&(pChDef->ChannelName),
                                  (PSECEXITDATA)&(pParms->ExitUserArea),
                                  (PCONTEXTDATA)&(pChDef->SecurityUserData));
              EXPECTSECMSG(pParms->ExitResponse,
                           ((PSECEXITDATA)&(pParms->ExitUserArea)),
                           DCE_SEC_INIT_SEC);
              break;

      case MQXR_INIT_SEC:
              /* if necessary and possible check target name against partner name in exit parameters */
              pParms->ExitResponse =
                 compareMQNames(&(pParms->ExitData[0]),
                                &(pParms->PartnerName[0]),
                                pChDef->ChannelType,
                                (PMQBYTE)&(pChDef->ChannelName));
              if ( pParms->ExitResponse == MQXCC_OK )
              {
                pParms->ExitResponse =
                   savePartnerName(&(pSecData->PartnerGSSName),
                                   (PMQCHAR48)&(pParms->PartnerName),
                                   (PMQBYTE)&(pChDef->ChannelName));
                if ( pParms->ExitResponse == MQXCC_OK )
                {
                   pParms->ExitResponse =
                      create_cred(pChDef,
                                  &(pSecData->credentialHandle),
                                  &(pContext->contextHandle));
                   if ( pParms->ExitResponse == MQXCC_OK )
                   {
                      initSecContext(pParms,
                                     pSecData,
                                     pContext,
                                     NULL,
                                     0,
                                     (PMQBYTE *)pExitBufferAddr,
                                     pExitBufferLength,
                                     (PMQBYTE)&(pChDef->ChannelName));
                      EXPECTSECMSG(pParms->ExitResponse,
                                   pSecData,
                                   DCE_SEC_ACC_SEC);
                   } /* endif */
                } /* endif */
              } /* endif */

              break;

      case MQXR_SEC_MSG:

              processSecurityMessage(pParms, pChDef,
                                     pAgentBuffer,
                                     *pDataLength,
                                     pExitBufferAddr,
                                     pExitBufferLength);
              break;

      case MQXR_TERM:
              terminateSecurityExit(pParms,
                                    (PSECEXITDATA)&(pParms->ExitUserArea),
                                    (PCONTEXTDATA)&(pChDef->SecurityUserData));
              break;

      default:
        /* Unrecognized exit function */

        /* Diagnostic output here */

        pParms->ExitResponse = MQXCC_SUPPRESS_FUNCTION;

   } /* endswitch */

   /* If OS/2: turn this thread back into a system thread */
   #if MQAT_DEFAULT==MQAT_OS2
     pthread_pseudo_thread_base_end();
   #endif

   if ( pParms->ExitResponse == MQXCC_SUPPRESS_FUNCTION )
   {
      pParms->ExitResponse2 = MQXR2_USE_AGENT_BUFFER;
   } else
   {
      if ( pParms->ExitResponse2 == MQXR2_USE_EXIT_BUFFER )
      {
         *pDataLength = *pExitBufferLength;
      } /* endif */
   } /* end if */

}


/**********************************************************************/
/*                                                                    */
/* Function Name: DCE_SEC_SRM_CHANNELEXIT                             */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* Function:                                                          */
/*           Main entry point for channel message encryption and      */
/*           decryption: acts as a send exit, a receive exit or       */
/*           a message exit.                                          */
/*                                                                    */
/* Input Parameters: pAgentBufferLength  UNUSED                       */
/*                                                                    */
/* Output Parameters: None                                            */
/*                                                                    */
/* InOut Parameters: pChannelExitParms   Channel exit parameter block */
/*                   pChannelDefinition  Channel definition           */
/*                   pDataLength         Length of data               */
/*                   pAgentBuffer        Agent buffer                 */
/*                   pExitBufferLength   Length of exit buffer        */
/*                   pExitBufferAddr     Address of exit buffer       */
/*                                                                    */
/* Returns: None                                                      */
/*                                                                    */
/**********************************************************************/

void MQENTRY DCE_SEC_SRM_CHANNELEXIT (
     PMQVOID  pChannelExitParms,   /* Channel exit parameter block */
     PMQVOID  pChannelDefinition,  /* Channel definition           */
     PMQLONG  pDataLength,         /* Length of data               */
     PMQLONG  pAgentBufferLength,  /* Length of agent buffer       */
     PMQVOID  pAgentBuffer,        /* Agent buffer                 */
     PMQLONG  pExitBufferLength,   /* Length of exit buffer        */
     PMQPTR   pExitBufferAddr)     /* Address of exit buffer       */
{
   PMQCXP        pParms;
   PMQCD         pChDef;
   PCONTEXTDATA  pContext;

   OM_uint32              major_status = 0;
   OM_uint32              minor_status = 0;
   gss_buffer_desc        input_buffer;
   gss_buffer_desc        output_buffer = { 0, NULL };
   PMQBYTE                msg = (PMQBYTE)pAgentBuffer;
   int                    conf_state;

   pParms   = (PMQCXP)pChannelExitParms;
   pChDef   = (PMQCD)pChannelDefinition;
   pContext = (PCONTEXTDATA)&(pChDef->SecurityUserData);

   if ( !(SENDEXITCALL) && !(RECEIVEEXITCALL) && !(MSGEXITCALL) )
   {
     /* Diagnostic output here */

     pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
     pParms->ExitResponse2 = MQXR2_USE_AGENT_BUFFER;
     return;
   }
   else
   {
      if ( (pExitBufferAddr != NULL) && (*pExitBufferAddr != NULL) )
      {
      /* free buffer used to pass back message on previous exit call */

         free(*pExitBufferAddr);
         *pExitBufferAddr = NULL;
         *pExitBufferLength = 0;
      } /* endif */

      if ( (pParms->ExitReason == MQXR_INIT) ||
           (pParms->ExitReason == MQXR_TERM) ) {
         pParms->ExitResponse = MQXCC_OK;
         return;
      } /* endif */
   } /* endif */

   /*  Now test for a receive exit call with byte 9 non-zero.            */
   /*  The data has not been encrypted by a                              */
   /*  matching send exit; this can                                      */   
   /*  happen quite validly during security context set up.              */
   /*  Such data is returned to the exit caller without further processing*/

     if ( RECEIVEEXITCALL && (msg[8] != '\0') ) /* zero if from send exit */
     {
     /* ExitResponse2 is passed in as MQXR2_USE_AGENT_BUFFER             */
     /* and the DataLength is unchanged.                                 */ 
       pParms->ExitResponse = MQXCC_OK;
       return;
     }

   if ( pContext->mechanism != DCEGSS_MECHANISM )

   { /*  Correct security exit has not been run */

   /* A send exit can validly be called before the security context is   */
   /* established. Clearly we cannot encrypt without a security context. */
   /* Just return the data to the exit caller unchanged.                 */

     if ( SENDEXITCALL )
     {
       *pExitBufferLength = *pDataLength;
       *pExitBufferAddr = pAgentBuffer;
       pParms->ExitResponse = MQXCC_OK;
       pParms->ExitResponse2 = MQXR2_USE_EXIT_BUFFER;
       return;
     }

     /* Diagnostic output here */

     pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
     pParms->ExitResponse2 = MQXR2_USE_AGENT_BUFFER;
     return;
   } /* endif */

   /* If OS/2: turn this thread into a pthread */
   #if MQAT_DEFAULT==MQAT_OS2
     pthread_pseudo_thread_base_start();
   #endif

   if ( SENDEXITCALL ||
        ( MSGEXITCALL && ( (pChDef->ChannelType == MQCHT_SENDER) ||
                           (pChDef->ChannelType == MQCHT_SERVER) ) ) )

   {  /* we are sending */

      input_buffer.length =
           (SENDEXITCALL) ? *pDataLength-8 : *pDataLength-sizeof(MQXQH);
      input_buffer.value  = &msg[(SENDEXITCALL) ? 8 : sizeof(MQXQH)];

      major_status = gss_seal(&minor_status, pContext->contextHandle,
                              TRUE, GSS_C_QOP_DEFAULT,
                              &input_buffer, &conf_state,
                              &output_buffer);

      if ( major_status == GSS_S_COMPLETE )
      {
       /* send exit header size is 10: 8 bytes of actual header, the          */
       /* 9th byte set to zero to indicate that                               */
       /* the data has been encrypted by the send exit                        */
       /* (9th byte is never zero unless set to zero in an exit),             */
       /* 10th byte is unused to provide the correct byte alignment for htons */
         composeSecMsg(pParms,
                       (PMQBYTE *)pExitBufferAddr,
                       pExitBufferLength,
                       pAgentBuffer,
                       (SENDEXITCALL) ? 10 : sizeof(MQXQH),
                       DCE_SEC_SEAL,
                       output_buffer.value,
                       output_buffer.length,
                       MQXCC_OK,
                       (PMQBYTE)&(pChDef->ChannelName));

         gss_release_buffer ( &minor_status, &output_buffer );
      }
      else
      {                       /* DCE call failed */

        /* Diagnostic output here */

        pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
      }
   }
   else   /* we are receiving */
   {
   /* offset is 10 for receive exit: header plus 9th byte set to zero by send   */
   /* exit plus 10th byte reserved to provide correct                           */
   /* byte alignment for htons on send                                          */
      MQLONG     offset    = (RECEIVEEXITCALL) ? 10 : sizeof(MQXQH);
      PDCESECMSG pDCE_secMsg = (PDCESECMSG)&msg[offset];

      if ( *pDataLength > (offset + (long)sizeof(pDCE_secMsg->header)) )
      {
         if ( (DCE_SECMECHANISM(pDCE_secMsg->header.mechanism)) &&
              (pDCE_secMsg->header.msgType == DCE_SEC_SEAL) )
         {
            int              qop_state;
            gss_buffer_desc  input_buffer;

            input_buffer.length = *pDataLength-offset-sizeof(pDCE_secMsg->header);
            input_buffer.value  = &(pDCE_secMsg->msg);

            major_status = gss_unseal(&minor_status, pContext->contextHandle,
                                      &input_buffer, &output_buffer,
                                      &conf_state, &qop_state);

            if ( major_status == GSS_S_COMPLETE )
            {
               PMQBYTE pToken;
               /* now remove bytes 9 and 10 from header     */
               MQLONG  headlen = (RECEIVEEXITCALL) ? 8 : sizeof(MQXQH);

               pToken = (PMQBYTE)malloc((size_t)(output_buffer.length+headlen));

               if ( pToken != NULL )
               {
                  if ( offset > 0 )
                  {
                     memcpy(pToken, msg, (size_t)headlen);
                  }
                  memcpy(&pToken[headlen], output_buffer.value, output_buffer.length);

                  *pExitBufferAddr = pToken;
                  *pExitBufferLength = output_buffer.length + headlen;
                  pParms->ExitResponse = MQXCC_OK;
                  pParms->ExitResponse2 = MQXR2_USE_EXIT_BUFFER;

                  gss_release_buffer ( &minor_status, &output_buffer );
               }
               else    /* malloc didn't work */
               {
                 /* Diagnostic output here */

                 pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
               } /* endif malloc worked */
            }
            else
            {                       /* DCE call failed */

              /* Diagnostic output here */

              pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
            }   /* endif ( major_status == GSS_S_COMPLETE ) */
         }
         else
         {
           /* Invalid message received */

           /* Diagnostic output here */

           pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
         } /* endif */
      }
      else
      {
        /* Invalid message length */

        /* Diagnostic output here */

        pParms->ExitResponse = MQXCC_CLOSE_CHANNEL;
      } /* endif */
   }

   /* If OS/2: turn this thread back into a system thread */
   #if MQAT_DEFAULT==MQAT_OS2
     pthread_pseudo_thread_base_end();
   #endif

   if ( pParms->ExitResponse == MQXCC_CLOSE_CHANNEL )
   {
      pParms->ExitResponse2 = MQXR2_USE_AGENT_BUFFER;
   }
   else
   {
      if ( pParms->ExitResponse2 == MQXR2_USE_EXIT_BUFFER )
      {
         *pDataLength = *pExitBufferLength;
      } /* endif */
   } /* end if */

}

/******************************************************************/
/* Known entry point to allow it to be loadable on UNIX           */
/******************************************************************/
void MQStart(void) { ; }
