 /* @(#) samples/c/amqsseta.c, samples, p520, p520-aux-CSD05G  1.17 00/10/16 12:10:35 */
 /********************************************************************/
 /*                                                                  */
 /* Program name: AMQSSETA                                           */
 /*                                                                  */
 /* Description: Sample C program using MQSET                        */
 /* <START_COPYRIGHT>                                                */
 /*  Statement:     Licensed Materials - Property of IBM             */
 /*                                                                  */
 /*                 04L1773, 5765-B73                                */
 /*                 04L1802, 5639-B42                                */
 /*                 04L1788, 5765-B74                                */
 /*                 04L1816, 5765-B75                                */
 /*                 04L1830, 5639-B43                                */
 /*                 (C) Copyright IBM Corp. 1994, 1998               */
 /* <END_COPYRIGHT>                                                  */
 /********************************************************************/
 /*                                                                  */
 /* Function:                                                        */
 /*                                                                  */
 /*                                                                  */
 /*   AMQSSETA is a sample C program that inhibits PUTs to a         */
 /*   message queue, and is an example of MQSET.  It is intended     */
 /*   to run as a triggered program, and so receives input in a      */
 /*   trigger parameter.                                             */
 /*                                                                  */
 /*   Note: one way to run AMQSSETA is to be running any             */
 /*         trigger monitor, and to use AMQSREQ* to send             */
 /*         request messages (containing queue names) to a           */
 /*         queue (eg SYSTEM.SAMPLE.SET) which must have been        */
 /*         defined to trigger AMQSSETA.                             */
 /*                                                                  */
 /*      -- gets request messages from a queue named in the          */
 /*         trigger message parameter, taking the content of each    */
 /*         message to be the name of a message queue                */
 /*                                                                  */
 /*      -- inhibits PUTs to the queue named in the request, and     */
 /*         sends a message to the named reply queue to confirm      */
 /*         it was done                                              */
 /*                                                                  */
 /*         alternatively, sends a report message if the queue       */
 /*         can't be inhibited - eg, if the queue to be inhibited    */
 /*         does not exist                                           */
 /*                                                                  */
 /*      -- stops when the input queue becomes empty                 */
 /*                                                                  */
 /*      -- writes a message for each MQI reason other than          */
 /*         MQRC_NONE; stops if there is a MQI completion code       */
 /*         of MQCC_FAILED                                           */
 /*                                                                  */
 /*                                                                  */
 /*   Program logic:                                                 */
 /*      MQCONNect to default queue manager                          */
 /*      MQOPEN message queue (A) for input                          */
 /*      while no MQI failures,                                      */
 /*      .  MQGET next message from queue A                          */
 /*      .  if request message received,                             */
 /*      .  .  OPEN queue (B) named in request for SET               */
 /*      .  .  MQSET, inhibit PUTs to queue B                        */
 /*      .  .  prepare reply message if PUT was inhibited            */
 /*      .  .  MQCLOSE queue B                                       */
 /*      .  .  prepare report message if reply not available         */
 /*      .  .  MQPUT1, send reply or report to named reply queue     */
 /*      MQCLOSE queue A                                             */
 /*      MQDISConnect from queue manager                             */
 /*                                                                  */
 /*                                                                  */
 /********************************************************************/
 /*                                                                  */
 /*   AMQSSETA has 1 parameter - a string (MQTMC2) based on the      */
 /*          initiation trigger message; only the QName and queue    */
 /*          manager name fields are used in this example            */
 /*                                                                  */
 /********************************************************************/
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
    /* includes for MQI  */
 #include <cmqc.h>

 int main(int argc, char **argv)
 {

   /*   Declare MQI structures needed                                */
   MQOD    odG = {MQOD_DEFAULT};    /* Object Descriptor for GET     */
   MQOD    odR = {MQOD_DEFAULT};    /* Object Descriptor for reply   */
   MQOD    odS = {MQOD_DEFAULT};    /* Object Descriptor for SET     */
   MQMD     md = {MQMD_DEFAULT};    /* Message Descriptor            */
   MQGMO   gmo = {MQGMO_DEFAULT};   /* get message options           */
   MQPMO   pmo = {MQPMO_DEFAULT};   /* put message options           */
      /** note, sample uses defaults where it can **/
   MQTMC2  *trig;                   /* trigger message structure     */

   MQHCONN  Hcon;                   /* connection handle             */
   MQHOBJ   Hobj;                   /* object handle, server queue   */
   MQHOBJ   Hset;                   /* handle for MQSET              */
   MQLONG   O_options;              /* MQOPEN options                */
   MQLONG   C_options;              /* MQCLOSE options               */
   MQLONG   CompCode;               /* completion code               */
   MQLONG   Reason;                 /* reason code                   */
   MQLONG   CReason;                /* reason codeon MQCONN          */
   char     buffer[101];            /* message buffer                */
   MQLONG   buflen;                 /* buffer length                 */
   MQLONG   messlen;                /* message length received       */
   MQLONG   Select[1];              /* attribute selector(s)         */
   MQLONG   IAV[1];                 /* integer attribute value(s)    */

   printf("Sample AMQSSETA start\n");
   if (argc < 2)
   {
     printf("Missing parameter - start program by MQI trigger\n");
     exit(99);
   }
   else if (argc > 2)
   {
     printf("Extra input parameters ignored\n");
   }

   /******************************************************************/
   /*                                                                */
   /*   Set the program argument into the trigger message            */
   /*   Check the struc_id of the trigger message.                   */ 
   /*                                                                */
   /******************************************************************/
   trig = (MQTMC2*) argv[1];        /* -> trigger message  */

   if (memcmp(trig -> StrucId
             ,MQTMC_STRUC_ID
             ,sizeof(MQCHAR4)) != 0)
   {
     printf("Invalid input trigger message provided\n");
     exit(99);
   }

   printf("%s\n", trig->QMgrName);

   /******************************************************************/
   /*                                                                */
   /*   This sample includes an explicit connect (MQCONN),           */
   /*   which isn't required on all MQSeries platforms,              */
   /*   because it's intended to be portable                         */
   /*                                                                */
   /******************************************************************/
   MQCONN(trig->QMgrName,          /* queue manager                  */
          &Hcon,                   /* connection handle              */
          &CompCode,               /* completion code                */
          &CReason);               /* reason code                    */

   /* report reason and stop if it failed     */
   if (CompCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %ld\n", CReason);
     exit(CReason);
   }

   /******************************************************************/
   /*                                                                */
   /*   Open the message queue for shared input                      */
   /*                                                                */
   /******************************************************************/
   memcpy(odG.ObjectName,          /* name of input queue            */
          trig->QName, MQ_Q_NAME_LENGTH);
   O_options = MQOO_INPUT_SHARED   /* open queue for shared input    */
         + MQOO_FAIL_IF_QUIESCING; /* but not if MQM stopping        */
   MQOPEN(Hcon,                    /* connection handle              */
          &odG,                    /* object descriptor for queue    */
          O_options,               /* open options                   */
          &Hobj,                   /* object handle                  */
          &CompCode,               /* completion code                */
          &Reason);                /* reason code                    */

   /* report reason if any; stop if it failed     */
   if (Reason != MQRC_NONE)
   {
     printf("MQOPEN (input) ended with reason code %ld\n", Reason);
   }

   if (CompCode == MQCC_FAILED)
   {
     exit(Reason);
   }

   /******************************************************************/
   /*                                                                */
   /*   Get messages from the message queue                          */
   /*   Loop until there is a warning or failure                     */
   /*                                                                */
   /******************************************************************/
   buflen = sizeof(buffer) - 1;
   gmo.Version = MQGMO_VERSION_2;    /* Avoid need to reset Message  */
   gmo.MatchOptions = MQMO_NONE;     /* ID and Correlation ID after  */
                                     /* every MQGET                  */
   gmo.Options = MQGMO_ACCEPT_TRUNCATED_MSG
                 + MQGMO_WAIT;     /* wait for new messages          */
   gmo.WaitInterval = 5000;        /* 5 second limit for waiting     */
   while (CompCode == MQCC_OK)
   {

     MQGET(Hcon,                /* connection handle                 */
           Hobj,                /* object handle                     */
           &md,                 /* message descriptor                */
           &gmo,                /* GET options                       */
           buflen,              /* buffer length                     */
           buffer,              /* message buffer                    */
           &messlen,            /* message length                    */
           &CompCode,           /* completion code                   */
           &Reason);            /* reason code                       */

     /* report reason if any (loop ends if it failed)   */
     if (Reason != MQRC_NONE)
     {
       printf("MQGET ended with reason code %ld\n", Reason);
     }

     /****************************************************************/
     /*                                                              */
     /*    Only process REQUEST messages                             */
     /*                                                              */
     /****************************************************************/
     if (CompCode == MQCC_OK)
     {
       buffer[messlen] = '\0';  /* end string ready to use */
       printf("%s\n", buffer);

       if (md.MsgType != MQMT_REQUEST)
       {
         printf("  -- not a request and discarded\n");
         continue;
       }

       /**************************************************************/
       /*                                                            */
       /*   Open named queue for SET                                 */
       /*                                                            */
       /**************************************************************/
       strncpy(odS.ObjectName,     /* name of queue from message     */
               buffer, MQ_Q_NAME_LENGTH);
       O_options = MQOO_SET        /* open to set attributes         */
                         + MQOO_FAIL_IF_QUIESCING;
       MQOPEN(Hcon,                /* connection handle              */
              &odS,                /* object descriptor for queue    */
              O_options,           /* open options                   */
              &Hset,               /* object handle for MQSET        */
              &CompCode,           /* completion code                */
              &Reason);            /* reason code                    */

       /* prepare to report error if it failed */
       if (CompCode != MQCC_OK)
       {
         md.MsgType = MQMT_REPORT;  /* flag that report is needed    */
         md.Feedback = Reason;      /* use Reason as the cause       */
       }

       /**************************************************************/
       /*                                                            */
       /*   Inhibit Put if the queue was opened successfully         */
       /*                                                            */
       /**************************************************************/
       else
       {
         Select[0] = MQIA_INHIBIT_PUT;   /* attribute selector       */
         IAV[0]    = MQQA_PUT_INHIBITED; /* attribute value          */
         MQSET(Hcon,            /* connection handle                 */
               Hset,            /* object handle                     */
               1L,              /* Selector count                    */
               Select,          /* Selector array                    */
               1L,              /* integer attribute count           */
               IAV,             /* integer attribute array           */
               0L,              /* character attribute count         */
               NULL,            /* character attribute array         */
               /*  note - can use NULL because count is zero         */
               &CompCode,       /* completion code                   */
               &Reason);        /* reason code                       */

         /************************************************************/
         /*                                                          */
         /*   Prepare reply if Inhibit (MQSET) worked, or            */
         /*   report if it failed                                    */
         /*                                                          */
         /************************************************************/
         if (CompCode == MQCC_OK)
         {
           strcat(buffer, " PUT inhibited");
           messlen  = strlen(buffer);             /* length of reply */
           md.MsgType = MQMT_REPLY;
         }
         else
         {
           md.MsgType = MQMT_REPORT;
           md.Feedback = Reason;                /* result of MQSET   */
         }

         /************************************************************/
         /*                                                          */
         /*   Close the queue opened for SET                         */
         /*   Result not checked in this example                     */
         /*                                                          */
         /************************************************************/
         C_options = 0;             /* no close options              */
         MQCLOSE(Hcon,              /* connection handle             */
                 &Hset,             /* object handle                 */
                 C_options,
                 &CompCode,         /* completion code               */
                 &Reason);          /* reason code                   */

       } /* end of sequence to inhibit PUT */

       /**************************************************************/
       /*                                                            */
       /*   If Inhibit (MQSET) was unsuccessful, prepare a report    */
       /*   message, suppressing data if not requested               */
       /*                                                            */
       /**************************************************************/
       if (md.MsgType & MQMT_REPORT)
       {                        /* if flagged as report needed ...   */
         if ((md.Report & MQRO_EXCEPTION_WITH_DATA)
                               != MQRO_EXCEPTION_WITH_DATA)
           messlen = 0;      /* suppress data if not wanted */
       }

       /**************************************************************/
       /*                                                            */
       /*   Send reply or report using MQPUT1                        */
       /*   (report is only to be sent if exceptions were requested) */
       /*                                                            */
       /**************************************************************/
       if ((md.MsgType == MQMT_REPLY)     /* if reply is available   */
          ||(md.Report & MQRO_EXCEPTION)) /* or exception requested  */
       {
         /************************************************************/
         /*                                                          */
         /*   Copy the ReplyTo queue name to the object descriptor   */
         /*                                                          */
         /************************************************************/
         strncpy(odR.ObjectName, md.ReplyToQ, MQ_Q_NAME_LENGTH);
         strncpy(odR.ObjectQMgrName,
                 md.ReplyToQMgr, MQ_Q_MGR_NAME_LENGTH);

         /************************************************************/
         /*                                                          */
         /*   MsgId and CorrelId are currently the values of the     */
         /*   got message.  Reset them if requested, then            */
         /*   stop further reports                                   */
         /*                                                          */
         /************************************************************/
         if ( !(md.Report & MQRO_PASS_CORREL_ID) )
         {
           memcpy(md.CorrelId, md.MsgId, sizeof(md.MsgId));
         }

         if ( !(md.Report & MQRO_PASS_MSG_ID) )
         {
           memcpy(md.MsgId, MQMI_NONE, sizeof(md.MsgId));
         }

         md.Report = MQRO_NONE;           /*    stop further reports */

         /************************************************************/
         /*                                                          */
         /*   Put the message                                        */
         /*                                                          */
         /************************************************************/
         MQPUT1(Hcon,            /* connection handle                */
                &odR,            /* object descriptor                */
                &md,             /* object descriptor for queue      */
                &pmo,            /* default options                  */
                messlen,         /* message length                   */
                buffer,          /* message buffer                   */
                &CompCode,       /* completion code                  */
                &Reason);        /* reason code                      */

         /* report reason if any  */
         if (Reason != MQRC_NONE)
         {
           printf("MQPUT1 ended with reason code %ld\n", Reason);
         }

       }  /* end sequence to send reply or report */
     }  /* end processing for message received */
   }  /* end while (CompCode == MQCC_OK) - message loop */

   /******************************************************************/
   /*                                                                */
   /*   Close server queue when no messages left                     */
   /*                                                                */
   /******************************************************************/
   C_options = 0;                   /* no close options              */
   MQCLOSE(Hcon,                    /* connection handle             */
           &Hobj,                   /* object handle                 */
           C_options,
           &CompCode,               /* completion code               */
           &Reason);                /* reason code                   */

   /* report reason, if any     */
   if (Reason != MQRC_NONE)
   {
     printf("MQCLOSE ended with reason code %ld\n", Reason);
   }

   /******************************************************************/
   /*                                                                */
   /*   Disconnect from MQM (unless previously connected)            */
   /*                                                                */
   /******************************************************************/
   if (CReason != MQRC_ALREADY_CONNECTED)
   {
     MQDISC(&Hcon,                   /* connection handle            */
            &CompCode,               /* completion code              */
            &Reason);                /* reason code                  */

     /* report reason, if any     */
     if (Reason != MQRC_NONE)
     {
       printf("MQDISC ended with reason code %ld\n", Reason);
     }
   }

   /******************************************************************/
   /*                                                                */
   /* END OF AMQSSETA                                                */
   /*                                                                */
   /******************************************************************/
   printf("Sample AMQSSETA end\n");
   return(0);
 }
