/* @(#) samples/c/xatm/amqsxag0.c, tranmgr, p520, p520-aux-CSD05G  1.20.1.1 00/11/15 13:26:21 */
/******************************************************************************/
/*                                                                            */
/* Program name: AMQSXAG0                                                     */
/*                                                                            */
/* Description:  Sample C program for MQ coordinating XA-compliant database   */
/*               managers.                                                    */
/*                                                                            */
/* Note:         Useful information and diagrams about this sample are in the */
/*               Application Programming Guide.                               */
/*                                                                            */
/* Statement:    Licensed Materials - Property of IBM                         */
/*                                                                            */
/*               04L1773, 5765-B73                                            */
/*               04L1802, 5639-B42                                            */
/*               04L1788, 5765-B74                                            */
/*               04L1816, 5765-B75                                            */
/*               04L1830, 5639-B43                                            */
/*                                                                            */
/*               (C) Copyright IBM Corp. 1998                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/*                                                                            */
/*    AMQSXAG0 is a sample C program for MQ to coordinate getting a message   */
/*    off a queue and updating two databases within an MQSeries unit of work. */
/*    This sample calls functions that are in AMQSXAB0.SQC and AMQSXAF0.SQC,  */
/*    these contain the SQL to update the two databases and hence must be     */
/*    prepared with the appropriate database.                                 */
/*                                                                            */
/*    Note: If you only have one XAResourceManager stanza in your QM.INI file */
/*          for a database program and hence are only concerned with one      */
/*          database within that database program, it is not necessary to     */
/*          issue the "EXEC SQL CONNECT TO XXXX;" statements in AMQSXAB0.SQC  */
/*          or AMQSXAF0.SQC, as the connection will be implicitly made for    */
/*          you. As anybody could modify the QM.INI file at sometime in the   */
/*          future, it is probably worth leaving in.                          */
/*                                                                            */
/*    -- A message is read from a queue (under sync point), it must be in     */
/*       the form:                                                            */
/*                                                                            */
/*       UPDATE Balance change=nnn WHERE Account=nnn                          */
/*                                                                            */
/*       The sample AMQSPUT can be used to put the messages on the queue.     */
/*                                                                            */
/*    -- Information from the database is obtained and updated with the       */
/*       information in the message.                                          */
/*                                                                            */
/*    -- The new status of the database is printed.                           */
/*                                                                            */
/* Program logic:                                                             */
/*                                                                            */
/*    MQCONN connect to default queue manager (or optionally supplied name)   */
/*    MQOPEN open queue for input (using supplied parameter)                  */
/*    while no failures                                                       */
/*    .  MQBEGIN start a unit of work                                         */
/*    .  MQGET get next message from queue under sync point                   */
/*    .  get information from databases                                       */
/*    .  update information from databases                                    */
/*    .  MQCMIT commit changes                                                */
/*    .  print updated information                                            */
/*    .  (no message available counts as failure, and loop ends)              */
/*    MQCLOSE close queue                                                     */
/*    MQDISC disconnect from queue manager                                    */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* AMQSXAG0 has 2 parameters - the name of the message queue (required)       */
/*                           - the queue manager name (optional)              */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* Includes                                                                   */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <cmqc.h>                                /* MQI                       */

/******************************************************************************/
/*                                                                            */
/* Defines                                                                    */
/*                                                                            */
/******************************************************************************/

#define OK      0                                /* define OK as zero         */
#define NOT_OK  1                                /* define NOT_OK as one      */

/******************************************************************************/
/*                                                                            */
/* Function prototypes for SQL routines in AMQSXAB0.SQC and AMQSXAF0.SQC      */
/*                                                                            */
/******************************************************************************/

int DeclareMQBankDBCursor(void);
int DeclareMQFeeDBCursor(void);
int ConnectToMQBankDB(void);
int ConnectToMQFeeDB(void);
int GetMQBankTBDetails(char *name, long account, long *balance,
                       long *transactions);
int GetMQFeeTBDetails(long account, long *feeDue, long *tranFee,
                      long *transactions);
int UpdateMQBankTBBalance(long balance, long transactions);
int UpdateMQFeeTBFeeDue(long feeDue, long transactions);


                                                 /*****************************/
int main(int argc, char *argv[])                 /* ----- START OF MAIN ----- */
                                                 /*****************************/
{
   /***************************************************************************/
   /* MQI structures                                                          */
   /***************************************************************************/
   MQOD od = {MQOD_DEFAULT};                     /* object descriptor         */
   MQMD md = {MQMD_DEFAULT};                     /* message descriptor        */
   MQGMO gmo = {MQGMO_DEFAULT};                  /* get message options       */
   MQBO bo = {MQBO_DEFAULT};                     /* begin options             */
   /***************************************************************************/
   /* MQI variables                                                           */
   /***************************************************************************/
   MQLONG rc=OK;                                 /* return code               */
   MQHCONN hCon;                                 /* handle to connection      */
   MQHOBJ hObj;                                  /* handle to object          */
   char QMName[50]="";                           /* default QM name           */
   MQLONG options;                               /* options                   */
   MQLONG reason;                                /* reason code               */
   MQLONG connReason;                            /* MQCONN reason code        */
   MQLONG compCode;                              /* completion code           */
   MQLONG openCompCode;                          /* MQOPEN completion code    */
   char msgBuf[100];                             /* message buffer            */
   MQLONG msgBufLen;                             /* message buffer length     */
   MQLONG msgLen;                                /* message length received   */
   /***************************************************************************/
   /* Other variables                                                         */
   /***************************************************************************/
   char *pStr;                                   /* ptr to string             */
   long balanceChange;                           /* balance change            */
   char name[40];                                /* name                      */
   long account;                                 /* account number            */
   long balance;                                 /* balance                   */
   long transactions;                            /* transactions              */
   long temp;                                    /* temporary variable        */
   long feeDue;                                  /* fee due                   */
   long tranFee;                                 /* transaction fee           */
   int gotMsg;                                   /* got message from queue    */
   int committedUpdate;                          /* committed update          */

   /***************************************************************************/
   /* First check we have been given correct arguments                        */
   /***************************************************************************/
   if (argc != 2 && argc != 3)
   {
      printf("Input is: %s 'queue name' 'queue manager name'.\n"
             "Note the queue manager name is optional\n", argv[0]);
      exit(99);
   } /* endif */

   if (argc == 3)                                /* use the queue manager     */
      strcpy(QMName, argv[2]);                   /* name supplied             */

   /***************************************************************************/
   /* Connect to queue manager                                                */
   /***************************************************************************/
   MQCONN(QMName, &hCon, &compCode, &connReason);

   if (compCode == MQCC_FAILED)
   {
     printf("MQCONN ended with reason code %li\n", connReason);
     exit((int) connReason);
   } /* endif */

   /***************************************************************************/
   /* Use input parameter as the name of the target queue                     */
   /***************************************************************************/
   strncpy(od.ObjectName, argv[1], (size_t) MQ_Q_NAME_LENGTH);
   printf("Target queue is %s\n", od.ObjectName);

   /***************************************************************************/
   /* Open the target message queue for input                                 */
   /***************************************************************************/
   options = MQOO_INPUT_AS_Q_DEF + MQOO_FAIL_IF_QUIESCING;

   MQOPEN(hCon, &od, options, &hObj, &openCompCode, &reason);

   if (reason != MQRC_NONE)
      printf("MQOPEN ended with reason code %li\n", reason);

   if (openCompCode == MQCC_FAILED)
   {
      printf("Unable to open queue for input\n");
      rc = openCompCode;                         /* stop further action       */
   } /* endif */

   /***************************************************************************/
   /* Declare the cursors for locking of reads from databases                 */
   /***************************************************************************/
   if (rc == OK)
      rc = DeclareMQBankDBCursor();
   if (rc == OK)
      rc = DeclareMQFeeDBCursor();

   /***************************************************************************/
   /* Set up some things for the MQGET                                        */
   /***************************************************************************/
   msgBufLen = sizeof(msgBuf) - 1;
   gmo.Version = MQGMO_VERSION_2;    /* Avoid need to reset message ID and    */
   gmo.MatchOptions = MQMO_NONE;     /* correlation ID after every MQGET      */
   gmo.Options = MQGMO_WAIT + MQGMO_CONVERT + MQGMO_SYNCPOINT;
   gmo.WaitInterval = 15000;                     /* 15 sec limit for waiting  */

   /***************************************************************************/
   /* Get messages from the message queue, loop until there is a failure      */
   /***************************************************************************/
   while (compCode != MQCC_FAILED && rc == OK)
   {
      /************************************************************************/
      /* Set flags so that we can back out if something goes wrong and not    */
      /* lose the message.                                                    */
      /************************************************************************/
      gotMsg = 0;                                /* set flag to FALSE         */
      committedUpdate = 0;                       /* set flag to FALSE         */

      /************************************************************************/
      /* Start a unit of work                                                 */
      /************************************************************************/
      MQBEGIN (hCon, &bo, &compCode, &reason);

      if (reason == MQRC_NONE)
      {
         printf("Unit of work started\n");
      }
      else
      {
         printf("MQBEGIN ended with reason code %li\n", reason);
         /*********************************************************************/
         /* If we get a reason code and only a warning on the compCode, there */
         /* is something wrong with one or more of the resource managers so   */
         /* stop looping and sort it out, whatever the compCode.              */
         /*********************************************************************/
         rc = NOT_OK;                            /* stop looping              */
      } /* endif */

      if (compCode == MQCC_FAILED)
         printf("Unable to start a unit of work\n");

      /************************************************************************/
      /* Get message off queue                                                */
      /************************************************************************/
      if (rc == OK)
      {

         MQGET(hCon, hObj, &md, &gmo, msgBufLen, msgBuf, &msgLen,
               &compCode, &reason);

         if (reason != MQRC_NONE)
         {
            if (reason == MQRC_NO_MSG_AVAILABLE)
            {
               printf("No more messages\n");
            }
            else
            {
               printf("MQGET ended with reason code %li\n", reason);

               if (reason == MQRC_TRUNCATED_MSG_FAILED)
                  compCode = MQCC_FAILED;        /* stop looping              */
            } /* endif */
         }
         else
         {
            gotMsg = 1;                          /* set flag to TRUE          */
         } /* endif */
      } /* endif */

      /************************************************************************/
      /* Process the message received                                         */
      /************************************************************************/
      if (compCode != MQCC_FAILED && rc == OK)
      {
         msgBuf[msgLen] = '\0';                  /* add string terminator     */

         pStr = strstr(msgBuf, "UPDATE Balance change=");
         if (pStr != NULL)
         {
            pStr += sizeof("UPDATE Balance change=") -1;
            sscanf(pStr, "%li", &balanceChange);
         }
         else
         {
            printf("Invalid string received: %s\n", msgBuf);
            rc = NOT_OK;                         /* stop looping              */
         } /* endif */

         if (rc == OK)
         {
            pStr = strstr(msgBuf, "Account=");
            if (pStr != NULL)
            {
               pStr += sizeof("Account=") -1;
               sscanf(pStr, "%li", &account);
            }
            else
            {
               printf("Invalid string received: %s\n", msgBuf);
               rc = NOT_OK;                      /* stop looping              */
            } /* endif */
         } /* endif */

         /*********************************************************************/
         /* Note only actively connected to one database at a time            */
         /*********************************************************************/
         /*********************************************************************/
         /* Get details from databases                                        */
         /*********************************************************************/
         if (rc == OK)
            rc = ConnectToMQBankDB();
         if (rc == OK)
            rc = GetMQBankTBDetails(name, account, &balance, &transactions);

         if (rc == OK)
            rc = ConnectToMQFeeDB();
         if (rc == OK)
            rc = GetMQFeeTBDetails(account, &feeDue, &tranFee, &temp);
         if (rc == OK)
         {
            /******************************************************************/
            /* The number of transactions to the two databases should be      */
            /* identical, stop if not.                                        */
            /******************************************************************/
            if (temp != transactions)
            {
               printf("Databases are out of step !\n");
               rc = NOT_OK;                      /* stop looping              */
            } /* endif */
         } /* endif */

         /*********************************************************************/
         /* Update the bank balance                                           */
         /*********************************************************************/
         if (rc == OK)
         {
            transactions++;                      /* bump no of transactions   */
            balance += balanceChange;            /* alter balance             */
            feeDue += tranFee;                   /* alter fee due             */

            rc = UpdateMQFeeTBFeeDue(feeDue, transactions);

            if (rc == OK)                        /* must now connect back to  */
               rc = ConnectToMQBankDB();         /* other database            */
            if (rc == OK)
               rc = UpdateMQBankTBBalance(balance, transactions);

            if (rc == OK)
            {
               printf("Account No %li Balance updated from %li to %li %s\n",
                      account, balance - balanceChange, balance, name);
               printf("Fee Due updated from %li to %li\n",
                      feeDue - tranFee, feeDue);

               /***************************************************************/
               /* We are going to commit the update so even if something goes */
               /* wrong now, the message has been used so don't back out.     */
               /***************************************************************/
               committedUpdate = 1;              /* set flag to TRUE          */

               /***************************************************************/
               /* Note: the cursor will be implicitly closed by the MQCMIT.   */
               /***************************************************************/
               MQCMIT(hCon, &compCode, &reason);

               if (reason == MQRC_NONE)
               {
                  printf("Unit of work successfully completed\n");
               }
               else
               {
                  printf("MQCMIT ended with reason code %li completion code "
                         "%li\n", reason, compCode);
                  rc = NOT_OK;                   /* stop looping              */
               } /* endif */
            } /* endif */
         } /* endif */
      } /* endif */

      /************************************************************************/
      /* If we got the message but something went wrong, back out so that we  */
      /* don't lose the message.                                              */
      /************************************************************************/
      if (gotMsg && !committedUpdate)
      {
         MQBACK(hCon, &compCode, &reason);

         if (reason == MQRC_NONE)
            printf("MQBACK successfully issued\n");
         else
            printf("MQBACK ended with reason code %li\n", reason);
      } /* endif */
   } /* endwhile */

   /***************************************************************************/
   /* Close queue if opened                                                   */
   /***************************************************************************/
   if (openCompCode != MQCC_FAILED)
   {
      options = 0;                               /* no close options          */

      MQCLOSE(hCon, &hObj, options, &compCode, &reason);

      if (reason != MQRC_NONE)
         printf("MQCLOSE ended with reason code %li\n", reason);
   } /* endif */

   /***************************************************************************/
   /* Disconnect from queue manager if not already connected                  */
   /***************************************************************************/
   if (connReason != MQRC_ALREADY_CONNECTED)
   {
      MQDISC(&hCon, &compCode, &reason);

      if (reason != MQRC_NONE)
         printf("MQDISC ended with reason code %li\n", reason);
   } /* endif */

   return 0;
                                                 /*****************************/
}                                                /* ------ END OF MAIN ------ */
                                                 /*****************************/
